@@ -1,93 +1,113 @@
 /***************************************************************************
- *   Copyright (C) 2008 by Hanna K.                                        *
- *   hanna_k@fmgirl.com                                                    *
+ *   Copyright (C) 2006-2008, 2014, 2016 by Hanna Knutsson                 *
+ *   hanna.knutsson@protonmail.com                                         *
  *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
+ *   This file is part of Eqonomize!.                                      *
+ *                                                                         *
+ *   Eqonomize! is free software: you can redistribute it and/or modify    *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation, either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
+ *   Eqonomize! is distributed in the hope that it will be useful,         *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with Eqonomize!. If not, see <http://www.gnu.org/licenses/>.    *
  ***************************************************************************/
 
-
 #ifndef EQONOMIZE_H
 #define EQONOMIZE_H
 
-#include <kapplication.h>
-#include <kxmlguiwindow.h>
-#include <kurl.h>
-#include <qvector.h>
-#include <qmap.h>
-#include <qdatetime.h>
-#include <QLabel>
+#include <QDateTime>
+#include <QDialog>
 #include <QDragEnterEvent>
 #include <QDropEvent>
-#include <kdialog.h>
-#include <qtextstream.h>
+#include <QLabel>
+#include <QMap>
 #include <QModelIndex>
+#include <QTextStream>
 #include <QTreeWidget>
 #include <QTreeWidgetItem>
+#include <QVector>
 #include <QWizardPage>
+#include <QUrl>
+#include <QApplication>
+#include <QMainWindow>
+
+#ifdef LOAD_EQZICONS_FROM_FILE
+	#ifdef RESOURCES_COMPILED
+		#define LOAD_APP_ICON(x) QIcon(ICON_DIR "/EQZ/apps/64x64/" x ".png")
+		/*#define LOAD_ICON(x) (QString(x).startsWith("eqz") ? QIcon(ICON_DIR "/EQZ/actions/64x64/" x ".png") : QIcon::fromTheme(x))
+		#define LOAD_ICON2(x, y) (QString(x).startsWith("eqz") ? QIcon(ICON_DIR "/EQZ/actions/64x64/" x ".png") : QIcon::fromTheme(x, y))*/
+		#define LOAD_ICON(x) QIcon(ICON_DIR "/EQZ/actions/64x64/" x ".png")
+		#define LOAD_ICON2(x, y) QIcon(ICON_DIR "/EQZ/actions/64x64/" y ".png")
+	#else
+		#define LOAD_APP_ICON(x) QIcon(ICON_DIR "/hicolor/64x64/apps/" x ".png")
+		#define LOAD_ICON(x) (QString(x).startsWith("eqz") ? QIcon(ICON_DIR "/hicolor/64x64/actions/" x ".png") : QIcon::fromTheme(x))
+		#define LOAD_ICON2(x, y) (QString(x).startsWith("eqz") ? QIcon(ICON_DIR "/hicolor/64x64/actions/" x ".png") : QIcon::fromTheme(x, QIcon::fromTheme(y)))
+	#endif
+#else
+	#define LOAD_APP_ICON(x) QIcon::fromTheme(x)
+	#define LOAD_ICON(x) QIcon::fromTheme(x)
+	#define LOAD_ICON2(x, y) QIcon::fromTheme(x, QIcon::fromTheme(y))
+#endif
 
-class Budget;
-class Transaction;
-class ScheduledTransaction;
-class SplitTransaction;
-class Account;
-class AssetsAccount;
-class IncomesAccount;
-class ExpensesAccount;
-class KLineEdit;
-class KDateEdit;
-class KComboBox;
-class QPushButton;
-class QLabel;
-class QCheckBox;
-class KPageWidget;
-class KPageWidgetItem;
-class QRadioButton;
-class TransactionListWidget;
-class KAction;
 class QAction;
-class KSelectAction;
-class KToggleAction;
-class KRecentFilesAction;
+class QActionGroup;
+class QCheckBox;
+class QLabel;
 class QMenu;
-class KMenu;
-class Expense;
-class Income;
-class Transfer;
-class ConfirmScheduleListViewItem;
-class EqonomizeValueEdit;
-class KTextEdit;
-class Security;
-class ReinvestedDividend;
-class SecurityTrade;
+class QPushButton;
+class QRadioButton;
 class QSpinBox;
 class QTabWidget;
-class EqonomizeMonthSelector;
-class KToolBar;
 class QVBoxLayout;
-class KAutoSaveFile;
+class QComboBox;
+class QLineEdit;
+class QCommandLineParser;
+class QToolBar;
+class QTabWidget;
+class QDateEdit;
+class QTextEdit;
+class QLocalSocket;
+class QLocalServer;
+class QPrinter;
+class QDialog;
+class QNetworkReply;
 
-typedef enum {
-	INITIAL_PERIOD_CURRENT_MONTH,
-	INITIAL_PERIOD_CURRENT_YEAR,
-	INITIAL_PERIOD_CURRENT_WHOLE_MONTH,
-	INITIAL_PERIOD_CURRENT_WHOLE_YEAR,
-	INITIAL_PERIOD_LAST
-} InitialPeriod;
+class CategoriesComparisonChart;
+class CategoriesComparisonReport;
+class CurrencyConversionDialog;
+class OverTimeChart;
+class OverTimeReport;
+class Account;
+class AssetsAccount;
+class CategoryAccount;
+class ExpensesAccount;
+class LoanAccount;
+class Budget;
+class ConfirmScheduleListViewItem;
+class EqonomizeMonthSelector;
+class EqonomizeValueEdit;
+class Expense;
+class ExpensesAccount;
+class Income;
+class IncomesAccount;
+class ReinvestedDividend;
+class ScheduledTransaction;
+class Security;
+class SecurityTrade;
+class SplitTransaction;
+class Transaction;
+class Transactions;
+class TransactionListWidget;
+class Transfer;
+class AccountComboBox;
 
-class Eqonomize : public KXmlGuiWindow {
+class Eqonomize : public QMainWindow {
 	
 	Q_OBJECT
 
@@ -96,7 +116,9 @@ class Eqonomize : public KXmlGuiWindow {
 		Eqonomize();
 		virtual ~Eqonomize();
 		
-		bool saveURL(const KUrl& url);
+		void sync(bool do_save = true, bool on_load = false, QWidget *parent = NULL);
+		bool saveURL(const QUrl& url, bool do_local_sync = true, bool do_cloud_sync = true, QWidget *parent = NULL);
+		bool saveAs(bool do_local_sync = true, bool do_cloud_sync = true, QWidget *parent = NULL);
 		bool askSave(bool before_exit = false);
 		void createDefaultBudget();
 		void readFileDependentOptions();
@@ -104,6 +126,10 @@ class Eqonomize : public KXmlGuiWindow {
 		Budget *budget;
 
 		bool first_run;
+		bool in_batch_edit;
+
+		void startBatchEdit();
+		void endBatchEdit();
 
 		void appendFilterExpense(Expense *expense, bool update_total_cost, bool update_accounts);
 		void appendFilterIncome(Income *income, bool update_total_income, bool update_accounts);
@@ -113,16 +139,17 @@ class Eqonomize : public KXmlGuiWindow {
 		void addScheduledTransactionValue(ScheduledTransaction *strans, bool update_value_display, bool subtract = false);
 		void subtractTransactionValue(Transaction *trans, bool update_value_display);
 		void addTransactionValue(Transaction *trans, const QDate &transdate, bool update_value_display, bool subtract = false, int n = -1, int b_future = -1, const QDate *monthdate = NULL);
-		void appendIncomesAccount(IncomesAccount *account);
-		void appendExpensesAccount(ExpensesAccount *account);
+		void appendIncomesAccount(IncomesAccount *account, QTreeWidgetItem *parent_item);
+		void appendExpensesAccount(ExpensesAccount *account, QTreeWidgetItem *parent_item);
 		void appendAssetsAccount(AssetsAccount *account);
+		void appendLoanAccount(LoanAccount *account);
 		void updateMonthlyBudget(Account *account);
 		void updateTotalMonthlyExpensesBudget();
 		void updateTotalMonthlyIncomesBudget();
 		bool editAccount(Account*);
 		bool editAccount(Account*, QWidget *parent);
 		void balanceAccount(Account*);
-		bool checkSchedule(bool update_display);
+		bool checkSchedule(bool update_display, QWidget *parent);
 		void updateScheduledTransactions();
 		void appendScheduledTransaction(ScheduledTransaction *strans);
 		bool editScheduledTransaction(ScheduledTransaction *strans);
@@ -135,9 +162,14 @@ class Eqonomize : public KXmlGuiWindow {
 		bool removeOccurrence(ScheduledTransaction *strans, const QDate &date);
 		bool newScheduledTransaction(int transaction_type, Security *security = NULL, bool select_security = false);
 		bool newScheduledTransaction(int transaction_type, Security *security, bool select_security, QWidget *parent, Account *account = NULL);
-		bool newSplitTransaction(QWidget *parent, AssetsAccount *account = NULL);
+		bool newExpenseWithLoan(QString description_value, double value_value, double quantity_value, QDate date_value, ExpensesAccount *category_value, QString payee_value, QString comment_value);
+		bool newExpenseWithLoan(QWidget *parent);
+		bool newMultiAccountTransaction(bool create_expenses, QString description_string, CategoryAccount *category_account, double quantity_value, QString comment_string);
+		bool newMultiAccountTransaction(QWidget *parent, bool create_expenses);
+		bool newMultiItemTransaction(QWidget *parent, AssetsAccount *account = NULL);
+		bool newDebtPayment(QWidget *parent, AssetsAccount *loan = NULL, bool only_interest = false);
 		bool editSplitTransaction(SplitTransaction *split);
-		bool editSplitTransaction(SplitTransaction *split, QWidget *parent);
+		bool editSplitTransaction(SplitTransaction *split, QWidget *parent, bool temporary_split = false);
 		bool splitUpTransaction(SplitTransaction *split);
 		bool removeSplitTransaction(SplitTransaction *split);
 		bool saveView(QTextStream &file, int fileformat);
@@ -149,8 +181,6 @@ class Eqonomize : public KXmlGuiWindow {
 		void updateSecurity(Security *security);
 		void updateSecurity(QTreeWidgetItem *i);
 		void updateSecurityAccount(AssetsAccount *account, bool update_display = true);
-		bool editReinvestedDividend(ReinvestedDividend *rediv, Security *security, QWidget *parent);
-		void editReinvestedDividend(ReinvestedDividend *rediv, Security *security);
 		bool editSecurityTrade(SecurityTrade *ts, QWidget *parent);
 		void editSecurityTrade(SecurityTrade *ts);
 		void setModified(bool has_been_modified = true);
@@ -158,38 +188,48 @@ class Eqonomize : public KXmlGuiWindow {
 		void showIncomes();
 		void showTransfers();
 		void updateSecuritiesStatistics();
-		bool crashRecovery(KUrl url);
-		bool newRefundRepayment(Transaction *trans);
+		bool crashRecovery(QUrl url);
+		bool newRefundRepayment(Transactions *trans);
 		void readOptions();
+		void setCommandLineParser(QCommandLineParser*);
+		bool timeToUpdateExchangeRates();
+		void addNewSchedule(ScheduledTransaction *strans, QWidget *parent);
 
 		QAction *ActionAP_1, *ActionAP_2, *ActionAP_3, *ActionAP_4, *ActionAP_5, *ActionAP_6, *ActionAP_7, *ActionAP_8;
-		KAction *ActionEditSchedule, *ActionEditOccurrence, *ActionDeleteSchedule, *ActionDeleteOccurrence;
-		KAction *ActionAddAccount, *ActionNewAssetsAccount, *ActionNewIncomesAccount, *ActionNewExpensesAccount, *ActionEditAccount, *ActionDeleteAccount, *ActionBalanceAccount;
-		KAction *ActionShowAccountTransactions;
-		KAction *ActionNewExpense, *ActionNewIncome, *ActionNewTransfer, *ActionNewSplitTransaction;
-		KAction *ActionEditTransaction, *ActionEditScheduledTransaction, *ActionEditSplitTransaction;
-		KAction *ActionJoinTransactions, *ActionSplitUpTransaction;
-		KAction *ActionDeleteTransaction, *ActionDeleteScheduledTransaction, *ActionDeleteSplitTransaction;
-		KAction *ActionNewSecurity, *ActionEditSecurity, *ActionBuyShares, *ActionSellShares, *ActionNewDividend, *ActionNewReinvestedDividend, *ActionNewSecurityTrade, *ActionSetQuotation, *ActionEditQuotations, *ActionEditSecurityTransactions, *ActionDeleteSecurity;
-		KAction *ActionNewRefund, *ActionNewRepayment, *ActionNewRefundRepayment;
-		KAction *ActionSave, *ActionFileReload, *ActionSaveView, *ActionPrintView;
-		KAction *ActionOverTimeReport, *ActionCategoriesComparisonReport, *ActionOverTimeChart, *ActionCategoriesComparisonChart;
-		KAction *ActionImportCSV, *ActionImportQIF, *ActionExportQIF;
-		KSelectAction *ActionSelectInitialPeriod;
-		KToggleAction *ActionExtraProperties;
-		KRecentFilesAction *ActionOpenRecent;
+		QAction *ActionEditSchedule, *ActionEditOccurrence, *ActionDeleteSchedule, *ActionDeleteOccurrence;
+		QAction *ActionAddAccount, *ActionNewAssetsAccount, *ActionNewLoan, *ActionNewIncomesAccount, *ActionNewExpensesAccount, *ActionEditAccount, *ActionDeleteAccount, *ActionCloseAccount, *ActionBalanceAccount, *ActionAddAccountMenu;
+		QAction *ActionShowAccountTransactions, *ActionShowLedger, *ActionReconcileAccount;
+		QAction *ActionNewExpense, *ActionNewIncome, *ActionNewTransfer, *ActionNewMultiItemTransaction;
+		QAction *ActionNewMultiAccountExpense, *ActionNewExpenseWithLoan, *ActionNewDebtPayment, *ActionNewDebtInterest;
+		QAction *ActionEditTransaction, *ActionEditScheduledTransaction, *ActionEditSplitTransaction;
+		QAction *ActionJoinTransactions, *ActionSplitUpTransaction;
+		QAction *ActionSelectAssociatedFile, *ActionOpenAssociatedFile;
+		QAction *ActionDeleteTransaction, *ActionDeleteScheduledTransaction, *ActionDeleteSplitTransaction;
+		QAction *ActionNewSecurity, *ActionEditSecurity, *ActionBuyShares, *ActionSellShares, *ActionNewDividend, *ActionNewReinvestedDividend, *ActionNewSecurityTrade, *ActionSetQuotation, *ActionEditQuotations, *ActionEditSecurityTransactions, *ActionDeleteSecurity;
+		QAction *ActionNewRefund, *ActionNewRepayment, *ActionNewRefundRepayment;
+		QAction *ActionFileNew, *ActionFileOpen, *ActionFileSave, *ActionFileSaveAs, *ActionFileReload, *ActionFileSync, *ActionSaveView, *ActionPrintView, *ActionPrintPreview, *ActionQuit;
+		QMenu *recentFilesMenu;
+		QList<QAction*> recentFileActionList;
+		QAction *ActionClearRecentFiles;
+		QAction *ActionOverTimeReport, *ActionCategoriesComparisonReport, *ActionOverTimeChart, *ActionCategoriesComparisonChart;
+		QAction *ActionImportCSV, *ActionImportQIF, *ActionImportEQZ, *ActionExportQIF;
+		QAction *ActionConvertCurrencies, *ActionUpdateExchangeRates;
+		QAction *ActionExtraProperties, *ActionUseExchangeRateForTransactionDate, *ActionSetBudgetPeriod, *AIPCurrentMonth, *AIPCurrentYear, *AIPCurrentWholeMonth, *AIPCurrentWholeYear, *AIPRememberLastDates, *ABFDaily, *ABFWeekly, *ABFFortnightly, *ABFMonthly, *ABFNever;
+		QAction *ActionSetMainCurrency, *ActionSyncSettings;
+		QActionGroup *ActionSelectInitialPeriod, *ActionSelectBackupFrequency;
+		QAction *ActionHelp, *ActionWhatsThis, *ActionReportBug, *ActionAbout, *ActionAboutQt;
 		
 	protected:
 
 		void setupActions();
+		void updateRecentFiles(QString filePath = QString());		
 		void saveOptions();
-		bool queryClose();
-		bool queryExit();
+		void closeEvent(QCloseEvent *event);
 		
 		void dragEnterEvent(QDragEnterEvent *event);
 		void dropEvent(QDropEvent *event);
 
-		KUrl current_url;
+		QUrl current_url;
 		double period_months, from_to_months;
 		bool modified, modified_auto_save, auto_save_timeout;
 		QDate from_date, to_date, frommonth_begin, prevmonth_begin;
@@ -197,36 +237,42 @@ class Eqonomize : public KXmlGuiWindow {
 		QDate prev_cur_date;
 		bool partial_budget;
 		bool b_extra;
-		KAutoSaveFile *cr_tmp_file;
+		QLocalSocket *socket;
+		QLocalServer *server;
+		QString cr_tmp_file;
 
-		KToolBar *scheduleButtons, *securitiesButtons;
-		KPageWidget *tabs;
+		QToolBar *fileToolbar, *accountsToolbar, *transactionsToolbar, *statisticsToolbar;
+		QTabWidget *tabs;
 		QTabWidget *accountsTabs;
 		QCheckBox *budgetButton;
 		EqonomizeValueEdit *budgetEdit;
 		EqonomizeMonthSelector *budgetMonthEdit;
 		QLabel *prevMonthBudgetLabel;
 		QWidget *accounts_page, *expenses_page, *incomes_page, *transfers_page, *securities_page, *schedule_page;
-		KPageWidgetItem *accounts_page_item, *expenses_page_item, *incomes_page_item, *transfers_page_item, *securities_page_item, *schedule_page_item;
 		QVBoxLayout *expensesLayout, *incomesLayout, *transfersLayout;
 		TransactionListWidget *expensesWidget, *incomesWidget, *transfersWidget;
 		QTreeWidget *accountsView, *securitiesView, *scheduleView;
-		QTreeWidgetItem *assetsItem, *incomesItem, *expensesItem;
+		QTreeWidgetItem *assetsItem, *liabilitiesItem, *incomesItem, *expensesItem;
 		QCheckBox *accountsPeriodFromButton;
-		KDateEdit *accountsPeriodFromEdit, *accountsPeriodToEdit;
+		QDateEdit *accountsPeriodFromEdit, *accountsPeriodToEdit;
 		QCheckBox *partialBudgetButton;
-		KComboBox *accountsPeriodCombo;
+		QComboBox *accountsPeriodCombo;
 		QCheckBox *securitiesPeriodFromButton;
-		KDateEdit *securitiesPeriodFromEdit, *securitiesPeriodToEdit;
+		QDateEdit *securitiesPeriodFromEdit, *securitiesPeriodToEdit;
 		QPushButton *newScheduleButton, *editScheduleButton, *removeScheduleButton;
 		QMenu *editScheduleMenu, *removeScheduleMenu;
 		QPushButton *newSecurityTransactionButton, *newSecurityButton, *setQuotationButton;
 		QLabel *securitiesStatLabel;
 		QLabel *footer1;
-
+		QCommandLineParser *parser;
+		QComboBox *setMainCurrencyCombo;
+		QNetworkReply *updateExchangeRatesReply, *checkVersionReply;
+		CurrencyConversionDialog *currencyConversionWindow;
+		
+		int prev_set_main_currency_index;
 		double total_value, total_cost, total_profit, total_rate;
-		double expenses_accounts_value, incomes_accounts_value, assets_accounts_value;
-		double expenses_accounts_change, incomes_accounts_change, assets_accounts_change;
+		double expenses_accounts_value, incomes_accounts_value, assets_accounts_value, liabilities_accounts_value;
+		double expenses_accounts_change, incomes_accounts_change, assets_accounts_change, liabilities_accounts_change;
 		double expenses_budget, expenses_budget_diff, incomes_budget, incomes_budget_diff;
 		QMap<Account*, double> account_value;
 		QMap<Account*, double> account_change;
@@ -241,33 +287,76 @@ class Eqonomize : public KXmlGuiWindow {
 		QMap<QTreeWidgetItem*, Account*> account_items;
 		QMap<Account*, QTreeWidgetItem*> item_accounts;
 
-		QMenu *accountPopupMenu, *securitiesPopupMenu, *schedulePopupMenu;
+		QMenu *assetsPopupMenu, *accountPopupMenu, *securitiesPopupMenu, *schedulePopupMenu;
+		
+		QDialog *helpDialog, *cccDialog, *ccrDialog, *otcDialog, *otrDialog, *syncDialog;
+		
+		QLineEdit *syncUploadEdit, *syncUrlEdit, *syncDownloadEdit;
+		QPushButton *uploadButton;
+		QCheckBox *syncAutoBox;
+		
+	protected slots:
+	
+		void checkAvailableVersion_readdata();
 
 	public slots:
 
 		void saveCrashRecovery();
 		void autoSave();
 		void onAutoSaveTimeout();
+		
+		void onActivateRequested(const QStringList&, const QString&);
 
 		void useExtraProperties(bool);
+		
+		void useExchangeRateForTransactionDate(bool);
+
+		void updateBudgetDay();
+		void setBudgetPeriod();
+		
+		void showFilter();
+		
+		void showHelp();
+		void reportBug();
+		void showAbout();
+		void showAboutQt();
 	
 		void importCSV();
 		void importQIF();
+		void importEQZ();
 		void exportQIF();
+		
+		void checkAvailableVersion();
+		
+		void checkExchangeRatesTimeOut();
+		void updateExchangeRates(bool do_currencies_modified = true);
+		void cancelUpdateExchangeRates();
+		void currenciesModified();
+		void warnAndAskForExchangeRate();
+		void setMainCurrency();
+		void setMainCurrencyIndexChanged(int index);
+		void updateUsesMultipleCurrencies();
+		void openCurrencyConversion();
+		
+		void cancelSync();
+		void fileSynchronize();
+		void openSynchronizationSettings();
+		void syncUploadChanged(const QString&);
+		void uploadClicked();
+		
+		void serverNewConnection();
+		void socketReadyRead();
 
 		void reloadBudget();
 
 		void showOverTimeReport();
-		void overTimeReportClosed();
 		void showCategoriesComparisonReport();
-		void overTimeChartClosed();
 		void showOverTimeChart();
-		void categoriesComparisonReportClosed();
 		void showCategoriesComparisonChart();
-		void categoriesComparisonChartClosed();
+		void printPreviewPaint(QPrinter*);
+		void showPrintPreview();
 		void printView();
 		void saveView();
-		void ledgerClosed();
 
 		void newSecurity();
 		void editSecurity();
@@ -286,7 +375,12 @@ class Eqonomize : public KXmlGuiWindow {
 		void popupSecuritiesMenu(const QPoint&);
 		void updateSecurities();
 		
-		void newSplitTransaction();
+		void newExpenseWithLoan();
+		void newMultiAccountExpense();
+		void newMultiAccountIncome();
+		void newMultiItemTransaction();
+		void newDebtPayment();
+		void newDebtInterest();
 		void newScheduledExpense();
 		void newScheduledIncome();
 		void newScheduledTransfer();
@@ -306,26 +400,33 @@ class Eqonomize : public KXmlGuiWindow {
 		void deleteSelectedSplitTransaction();
 		void joinSelectedTransactions();
 		void splitUpSelectedTransaction();
+		
+		void selectAssociatedFile();
+		void openAssociatedFile();
 
 		void newRefund();
 		void newRepayment();
 		void newRefundRepayment();
 
-		void onPageChange(KPageWidgetItem*, KPageWidgetItem*);
-		void onPageChange(QWidget*);
+		void onPageChange(int);
 
 		void showAccountTransactions(bool = false);
+		void showLedger();
+		void reconcileAccount();
 
 		void updateTransactionActions();
 		
-		void openURL(const KUrl&);
+		bool openURL(const QUrl&, bool merge = false);
 		void fileNew();
 		void fileOpen();
-		void fileOpenRecent(const KUrl&);
+		void fileOpenRecent(const QUrl&);
 		void fileReload();
-		void fileSave();
-		void fileSaveAs();
+		bool fileSave();
+		void onFilterSelected(QString);
+		bool fileSaveAs();
 		void optionsPreferences();
+		void clearRecentFiles();
+		void openRecent();
 
 		void checkSchedule();
 
@@ -334,14 +435,19 @@ class Eqonomize : public KXmlGuiWindow {
 		void popupAccountsMenu(const QPoint&);
 
 		void addAccount();
+		void accountAdded(Account *account);
+		void securityAdded(Security *security);
 		void newAssetsAccount();
-		void newIncomesAccount();
-		void newExpensesAccount();
+		void newLoan();
+		void newIncomesAccount(IncomesAccount *default_parent = NULL);
+		void newExpensesAccount(ExpensesAccount *default_parent = NULL);
 		void accountExecuted(QTreeWidgetItem*, int);
 		void accountExecuted(QTreeWidgetItem*);
+		void accountMoved(QTreeWidgetItem*, QTreeWidgetItem*);
 		void balanceAccount();
 		void editAccount();
 		void deleteAccount();
+		void closeAccount();
 		void accountsSelectionChanged();
 
 		void setPartialBudget(bool);
@@ -370,18 +476,10 @@ class Eqonomize : public KXmlGuiWindow {
 		void securitiesPrevYear();
 		void securitiesNextYear();
 		void securitiesCurrentYear();
-
-		void transactionAdded(Transaction*);
-		void transactionModified(Transaction*, Transaction*);
-		void transactionRemoved(Transaction*);
-
-		void scheduledTransactionAdded(ScheduledTransaction*);
-		void scheduledTransactionModified(ScheduledTransaction*, ScheduledTransaction*);
-		void scheduledTransactionRemoved(ScheduledTransaction*);
-		void scheduledTransactionRemoved(ScheduledTransaction*, ScheduledTransaction*);
-
-		void splitTransactionAdded(SplitTransaction*);
-		void splitTransactionRemoved(SplitTransaction*);
+		
+		void transactionAdded(Transactions*);
+		void transactionModified(Transactions*, Transactions*);
+		void transactionRemoved(Transactions*);
 
 		void filterAccounts();
 
@@ -394,7 +492,67 @@ class Eqonomize : public KXmlGuiWindow {
 
 };
 
-class ConfirmScheduleDialog : public KDialog {
+class OverTimeReportDialog : public QDialog {
+	
+	Q_OBJECT
+	
+	public:
+		
+		OverTimeReportDialog(Budget *budg, QWidget *parent);
+		OverTimeReport *report;
+
+	public slots:
+		
+		void reject();
+		
+};
+
+class CategoriesComparisonReportDialog : public QDialog {
+	
+	Q_OBJECT
+	
+	public:
+		
+		CategoriesComparisonReportDialog(bool extra_parameters, Budget *budg, QWidget *parent);
+		CategoriesComparisonReport *report;
+
+	public slots:
+		
+		void reject();
+		
+};
+
+class CategoriesComparisonChartDialog : public QDialog {
+	
+	Q_OBJECT
+		
+	public:
+		
+		CategoriesComparisonChartDialog(Budget *budg, QWidget *parent);		
+		CategoriesComparisonChart *chart;
+
+	public slots:
+		
+		void reject();
+		
+};
+
+class OverTimeChartDialog : public QDialog {
+	
+	Q_OBJECT
+		
+	public:
+		
+		OverTimeChartDialog(bool extra_parameters, Budget *budg, QWidget *parent);		
+		OverTimeChart *chart;
+
+	public slots:
+		
+		void reject();
+		
+};
+
+class ConfirmScheduleDialog : public QDialog {
 	
 	Q_OBJECT
 	
@@ -404,15 +562,14 @@ class ConfirmScheduleDialog : public KDialog {
 		Budget *budget;
 		bool b_extra;
 		QPushButton *editButton, *removeButton, *postponeButton;
-		ConfirmScheduleListViewItem *current_item;
 		int current_index;
 		
 	public:
 		
 		ConfirmScheduleDialog(bool extra_parameters, Budget *budg, QWidget *parent, QString title);
 
-		Transaction *firstTransaction();
-		Transaction *nextTransaction();
+		Transactions *firstTransaction();
+		Transactions *nextTransaction();
 
 	public slots:
 		
@@ -424,108 +581,26 @@ class ConfirmScheduleDialog : public KDialog {
 		
 };
 
-class EditAssetsAccountDialog : public KDialog {
-
-	Q_OBJECT
-	
-	protected:
-
-		KLineEdit *nameEdit;
-		EqonomizeValueEdit *valueEdit;
-		KTextEdit *descriptionEdit;
-		KComboBox *typeCombo;
-		QCheckBox *budgetButton;
-		Budget *budget;
-		Account *current_account;
-		
-	public:
-		
-		EditAssetsAccountDialog(Budget *budg, QWidget *parent, QString title);
-
-		AssetsAccount *newAccount();
-		void modifyAccount(AssetsAccount *account);
-		void setAccount(AssetsAccount *account);
-
-	protected slots:
-
-		void typeActivated(int);
-		void slotButtonClicked(int);
-		
-};
-
-class EditExpensesAccountDialog : public KDialog {
-
-	Q_OBJECT
-	
-	protected:
-
-		KLineEdit *nameEdit;
-		EqonomizeValueEdit *budgetEdit;
-		KTextEdit *descriptionEdit;
-		QCheckBox *budgetButton;
-		Budget *budget;
-		Account *current_account;
-		
-	public:
-		
-		EditExpensesAccountDialog(Budget *budg, QWidget *parent, QString title);
-		ExpensesAccount *newAccount();
-		void modifyAccount(ExpensesAccount *account);
-		void setAccount(ExpensesAccount *account);
-
-	protected slots:
-
-		void budgetEnabled(bool);
-		void slotButtonClicked(int);
-		
-};
-
-class EditIncomesAccountDialog : public KDialog {
-
-	Q_OBJECT
-	
-	protected:
-
-		KLineEdit *nameEdit;
-		EqonomizeValueEdit *budgetEdit;
-		KTextEdit *descriptionEdit;
-		QCheckBox *budgetButton;
-		Budget *budget;
-		Account *current_account;
-		
-	public:
-		
-		EditIncomesAccountDialog(Budget *budg, QWidget *parent, QString title);
-		IncomesAccount *newAccount();
-		void modifyAccount(IncomesAccount *account);
-		void setAccount(IncomesAccount *account);
-
-	protected slots:
-
-		void budgetEnabled(bool);
-		void slotButtonClicked(int);
-		
-};
-
-class EditSecurityDialog : public KDialog {
+class EditSecurityDialog : public QDialog {
 
 	Q_OBJECT
 
 	protected:
 
-		KLineEdit *nameEdit;
-		KTextEdit *descriptionEdit;
+		QLineEdit *nameEdit;
+		QTextEdit *descriptionEdit;
 		EqonomizeValueEdit *sharesEdit, *quotationEdit;
-		KDateEdit *quotationDateEdit;
-		KComboBox *typeCombo, *accountCombo;
-		QSpinBox *decimalsEdit;
+		QDateEdit *quotationDateEdit;
+		QComboBox *typeCombo;
+		AccountComboBox *accountCombo;
+		QSpinBox *decimalsEdit, *quotationDecimalsEdit;
 		QLabel *quotationLabel, *quotationDateLabel;
 		Budget *budget;
-		QVector<AssetsAccount*> accounts;
+		bool b_create_accounts;
 
 	public:
 		
-		EditSecurityDialog(Budget *budg, QWidget *parent, QString title);
+		EditSecurityDialog(Budget *budg, QWidget *parent, QString title, bool allow_account_creation = false);
 		Security *newSecurity();
 		bool modifySecurity(Security *security);
 		void setSecurity(Security *security);
@@ -534,10 +609,14 @@ class EditSecurityDialog : public KDialog {
 	protected slots:
 
 		void decimalsChanged(int);
+		void quotationDecimalsChanged(int);
+		void accountActivated(Account*);
+		void accept();
+		void newAccount();
 
 };
  
-class EditQuotationsDialog : public KDialog {
+class EditQuotationsDialog : public QDialog {
 
 	Q_OBJECT
 
@@ -546,12 +625,14 @@ class EditQuotationsDialog : public KDialog {
 		QLabel *titleLabel;
 		QTreeWidget *quotationsView;
 		EqonomizeValueEdit *quotationEdit;
-		KDateEdit *dateEdit;
+		QDateEdit *dateEdit;
 		QPushButton *changeButton, *addButton, *deleteButton;
+		int i_quotation_decimals;
+		Budget *budget;
 
 	public:
 
-		EditQuotationsDialog(QWidget *parent);
+		EditQuotationsDialog(Budget *budg, QWidget *parent);
 
 		void setSecurity(Security *security);
 		void modifyQuotations(Security *security);
@@ -565,69 +646,44 @@ class EditQuotationsDialog : public KDialog {
 
 };
 
-class RefundDialog : public KDialog {
+class RefundDialog : public QDialog {
 
 	Q_OBJECT
 
 	protected:
 
-		Transaction *transaction;
+		Transactions *transaction;
 		EqonomizeValueEdit *valueEdit, *quantityEdit;
-		KDateEdit *dateEdit;
-		KComboBox *accountCombo;
-		KLineEdit *commentsEdit;
+		QDateEdit *dateEdit;
+		QComboBox *accountCombo;
+		QLineEdit *commentsEdit;
 
 	public:
 
-		RefundDialog(Transaction *trans, QWidget *parent);
+		RefundDialog(Transactions *trans, QWidget *parent);
 
 		Transaction *createRefund();
 		bool validValues();
 
 	protected slots:
 		
-		void slotButtonClicked(int);
+		void accept();
+		void accountActivated(int);
 
 };
 
-class EditReinvestedDividendDialog : public KDialog {
+class EditSecurityTradeDialog : public QDialog {
 
 	Q_OBJECT
 
 	protected:
 
 		Budget *budget;
-		KComboBox *securityCombo;
-		EqonomizeValueEdit *sharesEdit;
-		KDateEdit *dateEdit;
-
-	public:
-
-		EditReinvestedDividendDialog(Budget *budg, Security *sec, bool select_security, QWidget *parent);
-
-		Security *selectedSecurity();
-		void securityChanged();
-		void setDividend(ReinvestedDividend *rediv);
-		bool modifyDividend(ReinvestedDividend *rediv);
-		ReinvestedDividend *createDividend();
-		bool validValues();
-
-	protected slots:
-		
-		void slotButtonClicked(int);
-
-};
-
-class EditSecurityTradeDialog : public KDialog {
-
-	Q_OBJECT
-
-	protected:
-
-		Budget *budget;
-		KComboBox *fromSecurityCombo, *toSecurityCombo;
-		EqonomizeValueEdit *valueEdit, *fromSharesEdit, *toSharesEdit;
-		KDateEdit *dateEdit;
+		QComboBox *fromSecurityCombo, *toSecurityCombo;
+		QLabel *sharesFromLabel, *shareToLabel;
+		EqonomizeValueEdit *fromSharesEdit, *toSharesEdit;
+		QDateEdit *dateEdit;
+		int prev_from_index;
 
 	public:
 
@@ -640,16 +696,16 @@ class EditSecurityTradeDialog : public KDialog {
 
 	protected slots:
 		
-		void slotButtonClicked(int);
+		void accept();
 		void maxShares();
 		Security *selectedFromSecurity();
 		Security *selectedToSecurity();
-		void fromSecurityChanged();
+		void fromSecurityChanged(bool = false);
 		void toSecurityChanged();
 
 };
 
-class SecurityTransactionsDialog : public KDialog {
+class SecurityTransactionsDialog : public QDialog {
 	
 	Q_OBJECT
 	
@@ -683,17 +739,21 @@ class EqonomizeTreeWidget : public QTreeWidget {
 	
 		EqonomizeTreeWidget(QWidget *parent);
 		EqonomizeTreeWidget();
+	
+	protected:
+	
+		void dropEvent(QDropEvent *event);
 
 	protected slots:
 	
 		void keyPressEvent(QKeyEvent*);
-		void onDoubleClick(const QModelIndex&);
 
 	signals:
 	
 		void returnPressed(QTreeWidgetItem*);
-		void doubleClicked(QTreeWidgetItem*, int);
-		 
+		void spacePressed(QTreeWidgetItem*);
+		void itemMoved(QTreeWidgetItem*, QTreeWidgetItem*);
+
 };
 
 class QIFWizardPage : public QWizardPage {
