@@ -1,120 +1,129 @@
 /***************************************************************************
- *   Copyright (C) 2008 by Hanna K.                                        *
- *   hanna_k@fmgirl.com                                                    *
+ *   Copyright (C) 2006-2008, 2014, 2016 by Hanna Knutsson                 *
+ *   hanna.knutsson@protonmail.com                                         *
  *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
+ *   This file is part of Eqonomize!.                                      *
+ *                                                                         *
+ *   Eqonomize! is free software: you can redistribute it and/or modify    *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation, either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
+ *   Eqonomize! is distributed in the hope that it will be useful,         *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with Eqonomize!. If not, see <http://www.gnu.org/licenses/>.    *
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-#include "recurrenceeditwidget.h"
+#include <QButtonGroup>
+#include <QBoxLayout>
+#include <QCheckBox>
+#include <QComboBox>
+#include <QGridLayout>
+#include <QGroupBox>
+#include <QHBoxLayout>
+#include <QLabel>
+#include <QLayout>
+#include <QListWidget>
+#include <QLineEdit>
+#include <QObject>
+#include <QPushButton>
+#include <QRadioButton>
+#include <QSpinBox>
+#include <QStackedWidget>
+#include <QVBoxLayout>
+#include <QDialogButtonBox>
+#include <QDateEdit>
+#include <QMessageBox>
+
 #include "budget.h"
 #include "recurrence.h"
+#include "recurrenceeditwidget.h"
 
-#include <kglobal.h>
-#include <klocale.h>
-#include <kiconloader.h>
-#include <kdeversion.h>
-#include <kstatusbar.h>
-#include <kconfig.h>
-#include <kurl.h>
-#include <QHBoxLayout>
-#include <QGridLayout>
-#include <QVBoxLayout>
-#include <QBoxLayout>
-#include <kmessagebox.h>
-#include <qlayout.h>
-#include <klineedit.h>
-#include <qcombobox.h>
-#include "kdateedit.h"
-#include <qlabel.h>
-#include <KDialogButtonBox>
-#include <kstdguiitem.h>
-#include <kstdaccel.h>
-#include <kaction.h>
-#include <kstandardaction.h>
-#include <qobject.h>
-#include <qpushbutton.h>
-#include <ktemporaryfile.h>
-#include <qlabel.h>
-#include <kseparator.h>
-#include <qcheckbox.h>
-#include <qradiobutton.h>
-#include <kpushbutton.h>
-#include <ktextedit.h>
-#include <QButtonGroup>
-#include <QStackedWidget>
-#include <qspinbox.h>
-#include <kcalendarsystem.h>
-#include <QListWidget>
-#include <QGroupBox>
+EditExceptionsDialog::EditExceptionsDialog(QWidget *parent)  : QDialog(parent, 0) {
 
-EditExceptionsDialog::EditExceptionsDialog(QWidget *parent)  : KDialog(parent, 0) {
-
-	setCaption(i18n("Edit Exceptions"));	
-	setButtons(KDialog::Ok | KDialog::Cancel);
-	setDefaultButton(KDialog::Cancel);
+	setWindowTitle(tr("Edit Exceptions"));
 	setModal(true);
-	
-	setMainWidget(new QWidget(this));
-	
-	QHBoxLayout *exceptionsLayout = new QHBoxLayout(mainWidget());
 
-	exceptionsList = new QListWidget(mainWidget());
-	exceptionsLayout->addWidget(exceptionsList);
+	QVBoxLayout *box1 = new QVBoxLayout(this);
+	
+	QGridLayout *layout = new QGridLayout();
+	box1->addLayout(layout);
+	
+	layout->addWidget(new QLabel(tr("Occurrences:"), this), 0, 0);
+	occurrencesList = new QListWidget(this);
+	occurrencesList->setSelectionMode(QListWidget::ExtendedSelection);
+	layout->addWidget(occurrencesList, 1, 0);
 	
 	QVBoxLayout *buttonsLayout = new QVBoxLayout();
-	exceptionsLayout->addLayout(buttonsLayout);
-	dateEdit = new KDateEdit(mainWidget());
-	dateEdit->setSizeAdjustPolicy(QComboBox::AdjustToContentsOnFirstShow);
-	dateEdit->setDate(QDate::currentDate());
-	buttonsLayout->addWidget(dateEdit);
-	addButton = new KPushButton(KStandardGuiItem::add(), mainWidget());
+	layout->addLayout(buttonsLayout, 0, 1, -1, 1);
+	buttonsLayout->addStretch(1);
+	addButton = new QPushButton(tr("Add Exception"), this);
+	addButton->setEnabled(false);
 	buttonsLayout->addWidget(addButton);
-	changeButton = new KPushButton(KStandardGuiItem::apply(), mainWidget());
-	changeButton->setEnabled(false);
-	buttonsLayout->addWidget(changeButton);
-	deleteButton = new KPushButton(KStandardGuiItem::del(), mainWidget());
+	deleteButton = new QPushButton(tr("Remove Exception"), this);
 	deleteButton->setEnabled(false);
 	buttonsLayout->addWidget(deleteButton);
 	buttonsLayout->addStretch(1);
 
+	layout->addWidget(new QLabel(tr("Exceptions:"), this), 0, 2);
+	exceptionsList = new QListWidget(this);
+	exceptionsList->setSelectionMode(QListWidget::ExtendedSelection);
+	layout->addWidget(exceptionsList, 1, 2);
+	
+	infoLabel = new QLabel(QString("<i>") + tr("Only the first fifty occurrences are shown.") + QString("</i>"), this);
+	infoLabel->hide();
+	layout->addWidget(infoLabel, 2, 0, 1, -1);
+
+	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
+	buttonBox->button(QDialogButtonBox::Cancel)->setShortcut(Qt::CTRL | Qt::Key_Return);
+	buttonBox->button(QDialogButtonBox::Cancel)->setDefault(true);
+	connect(buttonBox->button(QDialogButtonBox::Cancel), SIGNAL(clicked()), this, SLOT(reject()));
+	connect(buttonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), this, SLOT(accept()));
+	box1->addWidget(buttonBox);
+
+	connect(occurrencesList, SIGNAL(itemSelectionChanged()), this, SLOT(onSelectionChanged()));
 	connect(exceptionsList, SIGNAL(itemSelectionChanged()), this, SLOT(onSelectionChanged()));
 	connect(addButton, SIGNAL(clicked()), this, SLOT(addException()));
-	connect(changeButton, SIGNAL(clicked()), this, SLOT(changeException()));
 	connect(deleteButton, SIGNAL(clicked()), this, SLOT(deleteException()));
 
 }
 EditExceptionsDialog::~EditExceptionsDialog() {
 }
+
 void EditExceptionsDialog::setRecurrence(Recurrence *rec) {
+	occurrencesList->clear();
 	exceptionsList->clear();
 	if(rec) {
+		QDate next_date = rec->firstOccurrence();
+		int i = 0;
+		for(; i < 50 && next_date.isValid(); i++) {
+			occurrencesList->addItem(QLocale().toString(next_date, QLocale::ShortFormat));
+			next_date = rec->nextOccurrence(next_date);
+		}
+		if(i == 50) {
+			infoLabel->show();
+		} else {
+			infoLabel->hide();
+		}
 		for(QVector<QDate>::size_type i = 0; i < rec->exceptions.size(); i++) {
-			exceptionsList->addItem(KGlobal::locale()->formatDate(rec->exceptions[i], KLocale::ShortDate));
+			exceptionsList->addItem(QLocale().toString(rec->exceptions[i], QLocale::ShortFormat));
 		}
 		exceptionsList->sortItems();
 	}
+	o_rec = rec;
 	saveValues();
 }
 void EditExceptionsDialog::modifyExceptions(Recurrence *rec) {
 	for(int i = 0; i < exceptionsList->count(); i++) {
-		rec->addException(KGlobal::locale()->readDate(exceptionsList->item(i)->text()));
+		rec->addException(QLocale().toDate(exceptionsList->item(i)->text(), QLocale::ShortFormat));
 	}
 }
 bool EditExceptionsDialog::validValues() {
@@ -130,103 +139,100 @@ void EditExceptionsDialog::restoreValues() {
 	exceptionsList->clear();
 	exceptionsList->addItems(savedExceptions);
 }
-void EditExceptionsDialog::slotButtonClicked(int button) {
-	if(button == KDialog::Ok) {
-		if(!validValues()) return;
-		saveValues();
-	} else if(button == KDialog::Cancel) {
-		restoreValues();
-	}
-	KDialog::slotButtonClicked(button);
+void EditExceptionsDialog::accept() {
+	if(!validValues()) return;
+	saveValues();
+	QDialog::accept();
+}
+void EditExceptionsDialog::reject() {
+	restoreValues();
+	QDialog::reject();
 }
 void EditExceptionsDialog::onSelectionChanged() {
 	QList<QListWidgetItem*> list = exceptionsList->selectedItems();
-	if(!list.isEmpty()) {
-		changeButton->setEnabled(true);
-		deleteButton->setEnabled(true);
-		dateEdit->setDate(KGlobal::locale()->readDate(list.first()->text()));
-	} else {
-		changeButton->setEnabled(false);
-		deleteButton->setEnabled(false);
+	deleteButton->setEnabled(!list.isEmpty());
+	list = occurrencesList->selectedItems();
+	if(!list.isEmpty() && list.first() == occurrencesList->item(0)) {
+		occurrencesList->item(0)->setSelected(false);
+		list.removeFirst();
 	}
+	addButton->setEnabled(!list.isEmpty());
 }
 void EditExceptionsDialog::addException() {
-	QDate date = dateEdit->date();
-	if(!date.isValid()) {
-		KMessageBox::error(this, i18n("Invalid date."));
-		return;
+	QList<QListWidgetItem*> list = occurrencesList->selectedItems();
+	for(int i = 0; i < list.count(); i++) {
+		exceptionsList->addItem(list[i]->text());
+		delete list[i];
 	}
-	QString sdate = KGlobal::locale()->formatDate(date, KLocale::ShortDate);
-	QList<QListWidgetItem*> list = exceptionsList->findItems(sdate, Qt::MatchExactly);
-	if(list.isEmpty()) {
-		exceptionsList->addItem(sdate);
-		exceptionsList->sortItems();
-	}
-}
-void EditExceptionsDialog::changeException() {
-	QList<QListWidgetItem*> list = exceptionsList->selectedItems();
-	if(list.isEmpty()) return;
-	QDate date = dateEdit->date();
-	if(!date.isValid()) {
-		KMessageBox::error(this, i18n("Invalid date."));
-		return;
-	}
-	QString sdate = KGlobal::locale()->formatDate(date, KLocale::ShortDate);
-	list.first()->setText(sdate);
 	exceptionsList->sortItems();
 }
 void EditExceptionsDialog::deleteException() {
 	QList<QListWidgetItem*> list = exceptionsList->selectedItems();
-	if(list.isEmpty()) return;
-	delete list.first();
+	for(int i = 0; i < list.count(); i++) {
+		delete list[i];
+	}
+	modifyExceptions(o_rec);
+	occurrencesList->clear();
+	QDate next_date = o_rec->firstOccurrence();
+	for(int i = 0; i < 50 && next_date.isValid(); i++) {
+		occurrencesList->addItem(QLocale().toString(next_date, QLocale::ShortFormat));
+		next_date = o_rec->nextOccurrence(next_date);
+	}
 }
 
-EditRangeDialog::EditRangeDialog(const QDate &startdate, QWidget *parent) : KDialog(parent, 0), date(startdate) {
+EditRangeDialog::EditRangeDialog(const QDate &startdate, QWidget *parent) : QDialog(parent, 0), date(startdate) {
 
-	setCaption(i18n("Edit Recurrence Range"));	
-	setButtons(KDialog::Ok | KDialog::Cancel);
-	setDefaultButton(KDialog::Cancel);
+	setWindowTitle(tr("Edit Recurrence Range"));
 	setModal(true);
-	
-	setMainWidget(new QWidget(this));
-	QGridLayout *rangeLayout = new QGridLayout(mainWidget());
 
-	startLabel = new QLabel(i18n("Begins on: %1").arg(KGlobal::locale()->formatDate(startdate)), mainWidget());
+	QVBoxLayout *box1 = new QVBoxLayout(this);
+	
+	QGridLayout *rangeLayout = new QGridLayout();
+	box1->addLayout(rangeLayout);
+
+	startLabel = new QLabel(tr("Begins on: %1").arg(QLocale().toString(startdate)), this);
 	rangeLayout->addWidget(startLabel, 0, 0, 1, 3);
 
 	rangeButtonGroup = new QButtonGroup(this);
-	
-	foreverButton = new QRadioButton(i18n("No ending date"), mainWidget());
+
+	foreverButton = new QRadioButton(tr("No ending date"), this);
 	rangeButtonGroup->addButton(foreverButton);
 	rangeLayout->addWidget(foreverButton, 1, 0, 1, 3);
 
-	fixedCountButton = new QRadioButton(i18n("End after"), mainWidget());
+	fixedCountButton = new QRadioButton(tr("End after"), this);
 	rangeButtonGroup->addButton(fixedCountButton);
 	rangeLayout->addWidget(fixedCountButton, 2, 0);
-	fixedCountEdit = new QSpinBox(mainWidget());
+	fixedCountEdit = new QSpinBox(this);
 	fixedCountEdit->setRange(1, 9999);
 	rangeLayout->addWidget(fixedCountEdit, 2, 1);
-	rangeLayout ->addWidget(new QLabel(i18n("occurrence(s)"), mainWidget()), 2, 2);
+	rangeLayout ->addWidget(new QLabel(tr("occurrence(s)"), this), 2, 2);
 
-	endDateButton = new QRadioButton(i18n("End on"), mainWidget());
+	endDateButton = new QRadioButton(tr("End on"), this);
 	rangeButtonGroup->addButton(endDateButton);
 	rangeLayout->addWidget(endDateButton, 3, 0);
-	endDateEdit = new KDateEdit(mainWidget());
-	endDateEdit->setSizeAdjustPolicy(QComboBox::AdjustToContentsOnFirstShow);
+	endDateEdit = new QDateEdit(startdate, this);
+	endDateEdit->setCalendarPopup(true);
 	rangeLayout->addWidget(endDateEdit, 3, 1, 1, 2);
 
 	setRecurrence(NULL);
+	
+	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
+	buttonBox->button(QDialogButtonBox::Cancel)->setShortcut(Qt::CTRL | Qt::Key_Return);
+	buttonBox->button(QDialogButtonBox::Cancel)->setDefault(true);
+	connect(buttonBox->button(QDialogButtonBox::Cancel), SIGNAL(clicked()), this, SLOT(reject()));
+	connect(buttonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), this, SLOT(accept()));
+	box1->addWidget(buttonBox);
 
 	connect(fixedCountButton, SIGNAL(toggled(bool)), fixedCountEdit, SLOT(setEnabled(bool)));
 	connect(endDateButton, SIGNAL(toggled(bool)), endDateEdit, SLOT(setEnabled(bool)));
-			
+
 }
 
 EditRangeDialog::~EditRangeDialog() {
 }
 
 void EditRangeDialog::setStartDate(const QDate &startdate) {
-	startLabel->setText(i18n("Begins on: %1").arg(KGlobal::locale()->formatDate(startdate)));
+	startLabel->setText(tr("Begins on: %1").arg(QLocale().toString(startdate)));
 	date = startdate;
 	if(!endDateButton->isChecked() && date > endDateEdit->date()) {
 		endDateEdit->setDate(date);
@@ -266,14 +272,14 @@ QDate EditRangeDialog::endDate() {
 	}
 	return QDate();
 }
-void EditRangeDialog::slotButtonClicked(int button) {
-	if(button == KDialog::Ok) {
-		if(!validValues()) return;
-		saveValues();
-	} else if(button == KDialog::Cancel) {
-		restoreValues();
-	}
-	KDialog::slotButtonClicked(button);
+void EditRangeDialog::accept() {
+	if(!validValues()) return;
+	saveValues();
+	QDialog::accept();
+}
+void EditRangeDialog::reject() {
+	restoreValues();
+	QDialog::reject();
 }
 void EditRangeDialog::saveValues() {
 	fixedCountEdit_value = fixedCountEdit->value();
@@ -290,13 +296,15 @@ void EditRangeDialog::restoreValues() {
 bool EditRangeDialog::validValues() {
 	if(endDateButton->isChecked()) {
 		if(!endDateEdit->date().isValid()) {
-			KMessageBox::error(this, i18n("Invalid date."));
+			QMessageBox::critical(this, tr("Error"), tr("Invalid date."));
 			endDateEdit->setFocus();
+			endDateEdit->setCurrentSection(QDateTimeEdit::DaySection);
 			return false;
 		}
 		if(endDateEdit->date() < date) {
-			KMessageBox::error(this, i18n("End date before start date."));
+			QMessageBox::critical(this, tr("Error"), tr("End date before start date."));
 			endDateEdit->setFocus();
+			endDateEdit->setCurrentSection(QDateTimeEdit::DaySection);
 			return false;
 		}
 	}
@@ -307,23 +315,23 @@ bool EditRangeDialog::validValues() {
 RecurrenceEditWidget::RecurrenceEditWidget(const QDate &startdate, Budget *budg, QWidget *parent) : QWidget(parent), date(startdate), budget(budg) {
 
 	QVBoxLayout *recurrenceLayout = new QVBoxLayout(this);
-	recurrenceLayout->setContentsMargins(0, 0, 0, 0);
-	
-	recurrenceButton = new QCheckBox(i18n("Enable recurrance"), this);
+	//recurrenceLayout->setContentsMargins(0, 0, 0, 0);
+
+	recurrenceButton = new QCheckBox(tr("Enable recurrance"), this);
 	recurrenceLayout->addWidget(recurrenceButton);
 
-	ruleGroup = new QGroupBox(i18n("Recurrence Rule"), this);
+	ruleGroup = new QGroupBox(tr("Recurrence Rule"), this);
 	QVBoxLayout *ruleGroup_layout = new QVBoxLayout(ruleGroup);
-	
+
 	typeCombo = new QComboBox(ruleGroup);
 	typeCombo->setEditable(false);
-	typeCombo->addItem(i18n("Daily"));
-	typeCombo->addItem(i18n("Weekly"));
-	typeCombo->addItem(i18n("Monthly"));
-	typeCombo->addItem(i18n("Yearly"));
+	typeCombo->addItem(tr("Daily"));
+	typeCombo->addItem(tr("Weekly"));
+	typeCombo->addItem(tr("Monthly"));
+	typeCombo->addItem(tr("Yearly"));
 	typeCombo->setCurrentIndex(2);
 	ruleGroup_layout->addWidget(typeCombo);
-	
+
 	ruleStack = new QStackedWidget(ruleGroup);
 	ruleGroup_layout->addWidget(ruleStack);
 	QWidget *dailyRuleWidget = new QWidget(ruleStack);
@@ -338,29 +346,28 @@ RecurrenceEditWidget::RecurrenceEditWidget(const QDate &startdate, Budget *budg,
 	QVBoxLayout *dailyRuleLayout = new QVBoxLayout(dailyRuleWidget);
 	QHBoxLayout *dailyFrequencyLayout = new QHBoxLayout();
 	dailyRuleLayout->addLayout(dailyFrequencyLayout);
-	dailyFrequencyLayout->addWidget(new QLabel(i18n("Recur every"), dailyRuleWidget));
+	dailyFrequencyLayout->addWidget(new QLabel(tr("Recur every"), dailyRuleWidget));
 	dailyFrequencyEdit = new QSpinBox(dailyRuleWidget);
 	dailyFrequencyEdit->setRange(1, 9999);
 	dailyFrequencyLayout->addWidget(dailyFrequencyEdit);
-	dailyFrequencyLayout->addWidget(new QLabel(i18n("day(s)"), dailyRuleWidget));
+	dailyFrequencyLayout->addWidget(new QLabel(tr("day(s)"), dailyRuleWidget));
 	dailyFrequencyLayout->addStretch(1);
 	dailyRuleLayout->addStretch(1);
-	
+
 	QVBoxLayout *weeklyRuleLayout = new QVBoxLayout(weeklyRuleWidget);
 	QHBoxLayout *weeklyFrequencyLayout = new QHBoxLayout();
 	weeklyRuleLayout->addLayout(weeklyFrequencyLayout);
-	weeklyFrequencyLayout->addWidget(new QLabel(i18n("Recur every"), weeklyRuleWidget));
+	weeklyFrequencyLayout->addWidget(new QLabel(tr("Recur every"), weeklyRuleWidget));
 	weeklyFrequencyEdit = new QSpinBox(weeklyRuleWidget);
 	weeklyFrequencyEdit->setRange(1, 9999);
 	weeklyFrequencyLayout->addWidget(weeklyFrequencyEdit);
-	weeklyFrequencyLayout->addWidget(new QLabel(i18n("week(s) on:"), weeklyRuleWidget));
+	weeklyFrequencyLayout->addWidget(new QLabel(tr("week(s) on:"), weeklyRuleWidget));
 	weeklyFrequencyLayout->addStretch(1);
 	QHBoxLayout *weeklyDaysLayout = new QHBoxLayout();
 	weeklyRuleLayout->addLayout(weeklyDaysLayout);
-	int weekStart=KGlobal::locale()->weekStartDay();
-	const KCalendarSystem *calSys = KGlobal::locale()->calendar();
+	int weekStart = QLocale().firstDayOfWeek();	
 	for(int i = 0; i < 7; ++i ) {
-		weeklyButtons[i] = new QCheckBox(calSys->weekDayName(i + 1, KCalendarSystem::ShortDayName), weeklyRuleWidget);
+		weeklyButtons[i] = new QCheckBox(QDate::shortDayName(i + 1, QDate::StandaloneFormat), weeklyRuleWidget);
 	}
 	for(int i = weekStart - 1; i < 7; ++i ) {
 		weeklyDaysLayout->addWidget(weeklyButtons[i]);
@@ -373,82 +380,82 @@ RecurrenceEditWidget::RecurrenceEditWidget(const QDate &startdate, Budget *budg,
 	QVBoxLayout *monthlyRuleLayout = new QVBoxLayout(monthlyRuleWidget);
 	QHBoxLayout *monthlyFrequencyLayout = new QHBoxLayout();
 	monthlyRuleLayout->addLayout(monthlyFrequencyLayout);
-	monthlyFrequencyLayout->addWidget(new QLabel(i18n("Recur every"), monthlyRuleWidget));
+	monthlyFrequencyLayout->addWidget(new QLabel(tr("Recur every"), monthlyRuleWidget));
 	monthlyFrequencyEdit = new QSpinBox(monthlyRuleWidget);
 	monthlyFrequencyEdit->setRange(1, 9999);
 	monthlyFrequencyLayout->addWidget(monthlyFrequencyEdit);
-	monthlyFrequencyLayout->addWidget(new QLabel(i18n("month(s), after the start month"), monthlyRuleWidget));
+	monthlyFrequencyLayout->addWidget(new QLabel(tr("month(s), after the start month"), monthlyRuleWidget));
 	monthlyFrequencyLayout->addStretch(1);
 	QButtonGroup *monthlyButtonGroup = new QButtonGroup(this);
 	QGridLayout *monthlyButtonLayout = new QGridLayout();
 	monthlyRuleLayout->addLayout(monthlyButtonLayout, 1);
-	monthlyOnDayButton = new QRadioButton(i18n("Recur on the"), monthlyRuleWidget);
+	monthlyOnDayButton = new QRadioButton(tr("Recur on the"), monthlyRuleWidget);
 	monthlyOnDayButton->setChecked(true);
 	monthlyButtonGroup->addButton(monthlyOnDayButton);
 	monthlyButtonLayout->addWidget(monthlyOnDayButton, 0, 0);
 	QBoxLayout *monthlyDayLayout = new QHBoxLayout();
 	monthlyDayCombo = new QComboBox(monthlyRuleWidget);
-	monthlyDayCombo->addItem(i18n("1st"));
-	monthlyDayCombo->addItem(i18n("2nd"));
-	monthlyDayCombo->addItem(i18n("3rd"));
-	monthlyDayCombo->addItem(i18n("4th"));
-	monthlyDayCombo->addItem(i18n("5th"));
-	monthlyDayCombo->addItem(i18n("6th"));
-	monthlyDayCombo->addItem(i18n("7th"));
-	monthlyDayCombo->addItem(i18n("8th"));
-	monthlyDayCombo->addItem(i18n("9th"));
-	monthlyDayCombo->addItem(i18n("10th"));
-	monthlyDayCombo->addItem(i18n("11th"));
-	monthlyDayCombo->addItem(i18n("12th"));
-	monthlyDayCombo->addItem(i18n("13th"));
-	monthlyDayCombo->addItem(i18n("14th"));
-	monthlyDayCombo->addItem(i18n("15th"));
-	monthlyDayCombo->addItem(i18n("16th"));
-	monthlyDayCombo->addItem(i18n("17th"));
-	monthlyDayCombo->addItem(i18n("18th"));
-	monthlyDayCombo->addItem(i18n("19th"));
-	monthlyDayCombo->addItem(i18n("20th"));
-	monthlyDayCombo->addItem(i18n("21st"));
-	monthlyDayCombo->addItem(i18n("22nd"));
-	monthlyDayCombo->addItem(i18n("23rd"));
-	monthlyDayCombo->addItem(i18n("24th"));
-	monthlyDayCombo->addItem(i18n("25th"));
-	monthlyDayCombo->addItem(i18n("26th"));
-	monthlyDayCombo->addItem(i18n("27th"));
-	monthlyDayCombo->addItem(i18n("28th"));
-	monthlyDayCombo->addItem(i18n("29th"));
-	monthlyDayCombo->addItem(i18n("30th"));
-	monthlyDayCombo->addItem(i18n("31st"));
-	monthlyDayCombo->addItem(i18n("Last"));
-	monthlyDayCombo->addItem(i18n("2nd Last"));
-	monthlyDayCombo->addItem(i18n("3rd Last"));
-	monthlyDayCombo->addItem(i18n("4th Last"));
-	monthlyDayCombo->addItem(i18n("5th Last"));
-	monthlyDayCombo->setMaxVisibleItems(7);
+	monthlyDayCombo->addItem(tr("1st"));
+	monthlyDayCombo->addItem(tr("2nd"));
+	monthlyDayCombo->addItem(tr("3rd"));
+	monthlyDayCombo->addItem(tr("4th"));
+	monthlyDayCombo->addItem(tr("5th"));
+	monthlyDayCombo->addItem(tr("6th"));
+	monthlyDayCombo->addItem(tr("7th"));
+	monthlyDayCombo->addItem(tr("8th"));
+	monthlyDayCombo->addItem(tr("9th"));
+	monthlyDayCombo->addItem(tr("10th"));
+	monthlyDayCombo->addItem(tr("11th"));
+	monthlyDayCombo->addItem(tr("12th"));
+	monthlyDayCombo->addItem(tr("13th"));
+	monthlyDayCombo->addItem(tr("14th"));
+	monthlyDayCombo->addItem(tr("15th"));
+	monthlyDayCombo->addItem(tr("16th"));
+	monthlyDayCombo->addItem(tr("17th"));
+	monthlyDayCombo->addItem(tr("18th"));
+	monthlyDayCombo->addItem(tr("19th"));
+	monthlyDayCombo->addItem(tr("20th"));
+	monthlyDayCombo->addItem(tr("21st"));
+	monthlyDayCombo->addItem(tr("22nd"));
+	monthlyDayCombo->addItem(tr("23rd"));
+	monthlyDayCombo->addItem(tr("24th"));
+	monthlyDayCombo->addItem(tr("25th"));
+	monthlyDayCombo->addItem(tr("26th"));
+	monthlyDayCombo->addItem(tr("27th"));
+	monthlyDayCombo->addItem(tr("28th"));
+	monthlyDayCombo->addItem(tr("29th"));
+	monthlyDayCombo->addItem(tr("30th"));
+	monthlyDayCombo->addItem(tr("31st"));
+	monthlyDayCombo->addItem(tr("Last"));
+	monthlyDayCombo->addItem(tr("2nd Last"));
+	monthlyDayCombo->addItem(tr("3rd Last"));
+	monthlyDayCombo->addItem(tr("4th Last"));
+	monthlyDayCombo->addItem(tr("5th Last"));
 	monthlyDayLayout->addWidget(monthlyDayCombo);
-	monthlyDayLayout->addWidget(new QLabel(i18n("day"), monthlyRuleWidget));
+	monthlyDayLayout->addWidget(new QLabel(tr("day"), monthlyRuleWidget));
 	monthlyWeekendCombo = new QComboBox(monthlyRuleWidget);
-	monthlyWeekendCombo->addItem(i18n("possibly on weekend"));
-	monthlyWeekendCombo->addItem(i18n("but before weekend"));
-	monthlyWeekendCombo->addItem(i18n("but after weekend"));
+	monthlyWeekendCombo->addItem(tr("possibly on weekend"));
+	monthlyWeekendCombo->addItem(tr("but before weekend"));
+	monthlyWeekendCombo->addItem(tr("but after weekend"));
+	monthlyWeekendCombo->addItem(tr("on nearest weekday"));
 	monthlyDayLayout->addWidget(monthlyWeekendCombo);
 	monthlyDayLayout->addStretch(1);
 	monthlyButtonLayout->addLayout(monthlyDayLayout, 0, 1);
-	monthlyOnDayOfWeekButton = new QRadioButton(i18n("Recur on the"), monthlyRuleWidget);
+	monthlyOnDayOfWeekButton = new QRadioButton(tr("Recur on the"), monthlyRuleWidget);
 	monthlyButtonGroup->addButton(monthlyOnDayOfWeekButton);
 	monthlyButtonLayout->addWidget(monthlyOnDayOfWeekButton, 1, 0);
 	QBoxLayout *monthlyWeekLayout = new QHBoxLayout();
 	monthlyWeekCombo = new QComboBox(monthlyRuleWidget);
-	monthlyWeekCombo->addItem(i18n("1st"));
-	monthlyWeekCombo->addItem(i18n("2nd"));
-	monthlyWeekCombo->addItem(i18n("3rd"));
-	monthlyWeekCombo->addItem(i18n("4th"));
-	monthlyWeekCombo->addItem(i18n("5th"));
-	monthlyWeekCombo->addItem(i18n("Last"));
-	monthlyWeekCombo->addItem(i18n("2nd Last"));
-	monthlyWeekCombo->addItem(i18n("3rd Last"));
-	monthlyWeekCombo->addItem(i18n("4th Last"));
-	monthlyWeekCombo->addItem(i18n("5th Last"));
+	monthlyWeekCombo->addItem(tr("1st"));
+	monthlyWeekCombo->addItem(tr("2nd"));
+	monthlyWeekCombo->addItem(tr("3rd"));
+	monthlyWeekCombo->addItem(tr("4th"));
+	monthlyWeekCombo->addItem(tr("5th"));
+	monthlyWeekCombo->addItem(tr("Last"));
+	monthlyWeekCombo->addItem(tr("2nd Last"));
+	monthlyWeekCombo->addItem(tr("3rd Last"));
+	monthlyWeekCombo->addItem(tr("4th Last"));
+	monthlyWeekCombo->addItem(tr("5th Last"));
 	monthlyWeekLayout->addWidget(monthlyWeekCombo);
 	monthlyDayOfWeekCombo = new QComboBox(monthlyRuleWidget);
 	monthlyWeekLayout->addWidget(monthlyDayOfWeekCombo);
@@ -459,16 +466,16 @@ RecurrenceEditWidget::RecurrenceEditWidget(const QDate &startdate, Budget *budg,
 	QVBoxLayout *yearlyRuleLayout = new QVBoxLayout(yearlyRuleWidget);
 	QHBoxLayout *yearlyFrequencyLayout = new QHBoxLayout();
 	yearlyRuleLayout->addLayout(yearlyFrequencyLayout);
-	yearlyFrequencyLayout->addWidget(new QLabel(i18n("Recur every"), yearlyRuleWidget));
+	yearlyFrequencyLayout->addWidget(new QLabel(tr("Recur every"), yearlyRuleWidget));
 	yearlyFrequencyEdit = new QSpinBox(yearlyRuleWidget);
 	yearlyFrequencyEdit->setRange(1, 9999);
 	yearlyFrequencyLayout->addWidget(yearlyFrequencyEdit);
-	yearlyFrequencyLayout->addWidget(new QLabel(i18n("year(s), after the start year"), yearlyRuleWidget));
+	yearlyFrequencyLayout->addWidget(new QLabel(tr("year(s), after the start year"), yearlyRuleWidget));
 	yearlyFrequencyLayout->addStretch(1);
 	QButtonGroup *yearlyButtonGroup = new QButtonGroup(this);
 	QGridLayout *yearlyButtonLayout = new QGridLayout();
 	yearlyRuleLayout->addLayout(yearlyButtonLayout, 1);
-	yearlyOnDayOfMonthButton = new QRadioButton(i18nc("part before XXX of 'Recur on day XXX of month YYY'", "Recur on day"), yearlyRuleWidget);
+	yearlyOnDayOfMonthButton = new QRadioButton(tr("Recur on day", "part before XXX of 'Recur on day XXX of month YYY'"), yearlyRuleWidget);
 	yearlyButtonGroup->addButton(yearlyOnDayOfMonthButton);
 	yearlyOnDayOfMonthButton->setChecked(true);
 	yearlyButtonLayout->addWidget(yearlyOnDayOfMonthButton, 0, 0);
@@ -476,63 +483,65 @@ RecurrenceEditWidget::RecurrenceEditWidget(const QDate &startdate, Budget *budg,
 	yearlyDayOfMonthEdit = new QSpinBox(yearlyRuleWidget);
 	yearlyDayOfMonthEdit->setRange(1, 31);
 	yearlyMonthLayout->addWidget(yearlyDayOfMonthEdit);
-	yearlyMonthLayout->addWidget(new QLabel(i18nc("part between XXX and YYY of 'Recur on day XXX of month YYY'", "of"), yearlyRuleWidget));
+	yearlyMonthLayout->addWidget(new QLabel(tr("of", "part between XXX and YYY of 'Recur on day XXX of month YYY'"), yearlyRuleWidget));
 	yearlyMonthCombo = new QComboBox(yearlyRuleWidget);
 	yearlyMonthLayout->addWidget(yearlyMonthCombo);
 	yearlyWeekendCombo_month = new QComboBox(yearlyRuleWidget);
-	yearlyWeekendCombo_month->addItem(i18n("possibly on weekend"));
-	yearlyWeekendCombo_month->addItem(i18n("but before weekend"));
-	yearlyWeekendCombo_month->addItem(i18n("but after weekend"));
+	yearlyWeekendCombo_month->addItem(tr("possibly on weekend"));
+	yearlyWeekendCombo_month->addItem(tr("but before weekend"));
+	yearlyWeekendCombo_month->addItem(tr("but after weekend"));
+	yearlyWeekendCombo_month->addItem(tr("nearest weekend day"));
 	yearlyMonthLayout->addWidget(yearlyWeekendCombo_month);
 	yearlyMonthLayout->addStretch(1);
 	yearlyButtonLayout->addLayout(yearlyMonthLayout, 0, 1);
-	yearlyOnDayOfWeekButton = new QRadioButton(i18nc("Part before NNN in 'Recur on the NNN. WEEKDAY of MONTH'", "On the"), yearlyRuleWidget);
+	yearlyOnDayOfWeekButton = new QRadioButton(tr("On the", "Part before NNN in 'Recur on the NNN. WEEKDAY of MONTH'"), yearlyRuleWidget);
 	yearlyButtonGroup->addButton(yearlyOnDayOfWeekButton);
 	yearlyButtonLayout->addWidget(yearlyOnDayOfWeekButton, 1, 0);
 	QBoxLayout *yearlyWeekLayout = new QHBoxLayout();
 	yearlyWeekCombo = new QComboBox(yearlyRuleWidget);
-	yearlyWeekCombo->addItem(i18n("1st"));
-	yearlyWeekCombo->addItem(i18n("2nd"));
-	yearlyWeekCombo->addItem(i18n("3rd"));
-	yearlyWeekCombo->addItem(i18n("4th"));
-	yearlyWeekCombo->addItem(i18n("5th"));
-	yearlyWeekCombo->addItem(i18n("Last"));
-	yearlyWeekCombo->addItem(i18n("2nd Last"));
-	yearlyWeekCombo->addItem(i18n("3rd Last"));
-	yearlyWeekCombo->addItem(i18n("4th Last"));
-	yearlyWeekCombo->addItem(i18n("5th Last"));
+	yearlyWeekCombo->addItem(tr("1st"));
+	yearlyWeekCombo->addItem(tr("2nd"));
+	yearlyWeekCombo->addItem(tr("3rd"));
+	yearlyWeekCombo->addItem(tr("4th"));
+	yearlyWeekCombo->addItem(tr("5th"));
+	yearlyWeekCombo->addItem(tr("Last"));
+	yearlyWeekCombo->addItem(tr("2nd Last"));
+	yearlyWeekCombo->addItem(tr("3rd Last"));
+	yearlyWeekCombo->addItem(tr("4th Last"));
+	yearlyWeekCombo->addItem(tr("5th Last"));
 	yearlyWeekLayout->addWidget(yearlyWeekCombo);
 	yearlyDayOfWeekCombo = new QComboBox(yearlyRuleWidget);
 	yearlyWeekLayout->addWidget(yearlyDayOfWeekCombo);
-	yearlyWeekLayout->addWidget(new QLabel(i18nc("part between WEEKDAY and MONTH in 'Recur on NNN. WEEKDAY of MONTH'", "of"), yearlyRuleWidget));
+	yearlyWeekLayout->addWidget(new QLabel(tr("of", "part between WEEKDAY and MONTH in 'Recur on NNN. WEEKDAY of MONTH'"), yearlyRuleWidget));
 	yearlyMonthCombo_week = new QComboBox(yearlyRuleWidget);
 	yearlyWeekLayout->addWidget(yearlyMonthCombo_week);
 	yearlyWeekLayout->addStretch(1);
 	yearlyButtonLayout->addLayout(yearlyWeekLayout, 1, 1);
-	yearlyOnDayOfYearButton = new QRadioButton(i18n("Recur on day #"), yearlyRuleWidget);
+	yearlyOnDayOfYearButton = new QRadioButton(tr("Recur on day #"), yearlyRuleWidget);
 	yearlyButtonGroup->addButton(yearlyOnDayOfYearButton);
 	yearlyButtonLayout->addWidget(yearlyOnDayOfYearButton, 2, 0);
 	QBoxLayout *yearlyDayLayout = new QHBoxLayout();
 	yearlyDayOfYearEdit = new QSpinBox(yearlyRuleWidget);
 	yearlyDayOfYearEdit->setRange(1, 366);
 	yearlyDayLayout->addWidget(yearlyDayOfYearEdit);
-	yearlyDayLayout->addWidget(new QLabel(i18nc("part after NNN of 'Recur on day #NNN of the year'", " of the year"), yearlyRuleWidget));
+	yearlyDayLayout->addWidget(new QLabel(tr(" of the year", "part after NNN of 'Recur on day #NNN of the year'"), yearlyRuleWidget));
 	yearlyWeekendCombo_day = new QComboBox(yearlyRuleWidget);
-	yearlyWeekendCombo_day->addItem(i18n("possibly on weekend"));
-	yearlyWeekendCombo_day->addItem(i18n("but before weekend"));
-	yearlyWeekendCombo_day->addItem(i18n("but after weekend"));
+	yearlyWeekendCombo_day->addItem(tr("possibly on weekend"));
+	yearlyWeekendCombo_day->addItem(tr("but before weekend"));
+	yearlyWeekendCombo_day->addItem(tr("but after weekend"));
+	yearlyWeekendCombo_day->addItem(tr("nearest weekend day"));
 	yearlyDayLayout->addWidget(yearlyWeekendCombo_day);
 	yearlyDayLayout->addStretch(1);
 	yearlyButtonLayout->addLayout(yearlyDayLayout, 2, 1);
 	yearlyRuleLayout->addStretch(1);
 
 	recurrenceLayout->addWidget(ruleGroup);
-	
+
 	QHBoxLayout *buttonLayout = new QHBoxLayout();
 	recurrenceLayout->addLayout(buttonLayout);
-	rangeButton = new KPushButton(i18n("Range..."), this);
+	rangeButton = new QPushButton(tr("Range…"), this);
 	buttonLayout->addWidget(rangeButton);
-	exceptionsButton = new KPushButton(i18n("Exceptions..."), this);
+	exceptionsButton = new QPushButton(tr("Occurrences/Exceptions…"), this);
 	buttonLayout->addWidget(exceptionsButton);
 
 	recurrenceLayout->addStretch(1);
@@ -552,20 +561,19 @@ RecurrenceEditWidget::RecurrenceEditWidget(const QDate &startdate, Budget *budg,
 	int months = 0;
 	QDate date = QDate::currentDate();
 	for(int i = 0; i < 10; i++) {
-		int months2 = calSys->monthsInYear(date);
+		int months2 = 12;
 		if(months2 > months) {
-			int year = calSys->year(date);
 			for(int i2 = months + 1; i2 <= months2; i2++) {
-				yearlyMonthCombo_week->addItem(calSys->monthName(i2, year));
-				yearlyMonthCombo->addItem(calSys->monthName(i2, year));
+				yearlyMonthCombo_week->addItem(QDate::longMonthName(i2, QDate::StandaloneFormat));
+				yearlyMonthCombo->addItem(QDate::longMonthName(i2, QDate::StandaloneFormat));
 			}
 			months = months2;
 		}
-		date = calSys->addYears(date, 1);
+		date = date.addYears(1);
 	}
 	for(int i = 1; i <= 7; i++) {
-		yearlyDayOfWeekCombo->addItem(calSys->weekDayName(i));
-		monthlyDayOfWeekCombo->addItem(calSys->weekDayName(i));
+		yearlyDayOfWeekCombo->addItem(QDate::longDayName(i, QDate::StandaloneFormat));
+		monthlyDayOfWeekCombo->addItem(QDate::longDayName(i, QDate::StandaloneFormat));
 	}
 
 	connect(typeCombo, SIGNAL(activated(int)), ruleStack, SLOT(setCurrentIndex(int)));
@@ -574,15 +582,18 @@ RecurrenceEditWidget::RecurrenceEditWidget(const QDate &startdate, Budget *budg,
 	connect(recurrenceButton, SIGNAL(toggled(bool)), ruleGroup, SLOT(setEnabled(bool)));
 	connect(recurrenceButton, SIGNAL(toggled(bool)), rangeButton, SLOT(setEnabled(bool)));
 	connect(recurrenceButton, SIGNAL(toggled(bool)), exceptionsButton, SLOT(setEnabled(bool)));
-	
+
 }
 
 RecurrenceEditWidget::~RecurrenceEditWidget() {
 }
 
 void RecurrenceEditWidget::editExceptions() {
+	Recurrence *rec = createRecurrence();
+	exceptionsDialog->setRecurrence(rec);
 	exceptionsDialog->exec();
 	exceptionsDialog->hide();
+	delete rec;
 }
 void RecurrenceEditWidget::editRange() {
 	rangeDialog->exec();
@@ -728,37 +739,33 @@ bool RecurrenceEditWidget::validValues() {
 				}
 			}
 			if(!b) {
-				KMessageBox::error(this, i18n("No day of week selected for weekly recurrence."));
+				QMessageBox::critical(this, tr("Error"), tr("No day of week selected for weekly recurrence."));
 				weeklyButtons[0]->setFocus();
 				return false;
 			}
 			break;
 		}
 		case 2: {
-			if(KGlobal::locale()->calendarType() == "gregorian") {
-				int i_frequency = monthlyFrequencyEdit->value();
-				if(i_frequency % 12 == 0) {
-					const KCalendarSystem *calSys = KGlobal::locale()->calendar();
-					int i_dayofmonth = monthlyDayCombo->currentIndex() + 1;
-					int i_month = calSys->month(date);
-					if(i_dayofmonth <= 31 && ((i_month == 2 && i_dayofmonth > 29) || (i_dayofmonth > 30 && (i_month == 4 || i_month == 6 || i_month == 9 || i_month == 11)))) {
-						KMessageBox::error(this, i18n("Selected day will never occur with selected frequency and start date."));
-						monthlyDayCombo->setFocus();
-						return false;
-					}
+			int i_frequency = monthlyFrequencyEdit->value();
+			if(i_frequency % 12 == 0) {					
+				int i_dayofmonth = monthlyDayCombo->currentIndex() + 1;
+				int i_month = date.month();
+				if(i_dayofmonth <= 31 && ((i_month == 2 && i_dayofmonth > 29) || (i_dayofmonth > 30 && (i_month == 4 || i_month == 6 || i_month == 9 || i_month == 11)))) {
+					QMessageBox::critical(this, tr("Error"), tr("Selected day will never occur with selected frequency and start date."));
+					monthlyDayCombo->setFocus();
+					return false;
 				}
 			}
 			break;
 		}
-		case 3: {
-			const KCalendarSystem *calSys = KGlobal::locale()->calendar();
+		case 3: {			
 			if(yearlyOnDayOfMonthButton->isChecked()) {
 				int i_frequency = yearlyFrequencyEdit->value();
 				int i_dayofmonth = yearlyDayOfMonthEdit->value();
 				int i_month = yearlyMonthCombo->currentIndex() + 1;
-				if(KGlobal::locale()->calendarType() == "gregorian") {
+				if(IS_GREGORIAN_CALENDAR) {
 					if((i_month == 2 && i_dayofmonth > 29) || (i_dayofmonth > 30 && (i_month == 4 || i_month == 6 || i_month == 9 || i_month == 11))) {
-						KMessageBox::error(this, i18n("Selected day does not exist in selected month."));
+						QMessageBox::critical(this, tr("Error"), tr("Selected day does not exist in selected month."));
 						yearlyDayOfMonthEdit->setFocus();
 						return false;
 					} else if(i_month != 2 || i_dayofmonth < 29) {
@@ -766,35 +773,35 @@ bool RecurrenceEditWidget::validValues() {
 					}
 				}
 				QDate nextdate;
-				calSys->setYMD(nextdate, calSys->year(date), i_month, 1);
-				if(i_dayofmonth > calSys->daysInMonth(nextdate)) {
+				nextdate.setDate(date.year(), i_month, 1);
+				if(i_dayofmonth > nextdate.daysInMonth()) {
 					int i = 10;
 					do {
 						if(i == 0) {
-							KMessageBox::error(this, i18n("Selected day will never occur with selected frequency and start date."));
+							QMessageBox::critical(this, tr("Error"), tr("Selected day will never occur with selected frequency and start date."));
 							yearlyDayOfMonthEdit->setFocus();
 							return false;
 						}
-						nextdate = calSys->addYears(nextdate, i_frequency);
-						calSys->setYMD(nextdate, calSys->year(nextdate), i_month, 1);
+						nextdate = nextdate.addYears(i_frequency);
+						nextdate.setDate(nextdate.year(), i_month, 1);
 						i--;
-					} while(i_dayofmonth > calSys->daysInMonth(nextdate));
+					} while(i_dayofmonth > nextdate.daysInMonth());
 				}
 			} else if(yearlyOnDayOfYearButton->isChecked()) {
 				int i_frequency = yearlyFrequencyEdit->value();
 				int i_dayofyear = yearlyDayOfYearEdit->value();
-				if(i_dayofyear > calSys->daysInYear(date)) {
+				if(i_dayofyear > date.daysInYear()) {
 					QDate nextdate = date;
 					int i = 10;
 					do {
 						if(i == 0) {
-							KMessageBox::error(this, i18n("Selected day will never occur with selected frequency and start date."));
+							QMessageBox::critical(this, tr("Error"), tr("Selected day will never occur with selected frequency and start date."));
 							yearlyDayOfYearEdit->setFocus();
 							return false;
 						}
-						nextdate = calSys->addYears(nextdate, i_frequency);
+						nextdate = nextdate.addYears(i_frequency);
 						i--;
-					} while(i_dayofyear > calSys->daysInYear(nextdate));
+					} while(i_dayofyear > nextdate.daysInYear());
 				}
 			}
 			break;
@@ -810,5 +817,3 @@ void RecurrenceEditWidget::setStartDate(const QDate &startdate) {
 	rangeDialog->setStartDate(date);
 }
 
-#include "recurrenceeditwidget.moc"
-
